# Load data
source("functions.R")
dipperdata <- read.csv("Complete data set_with territory_with second clutch.txt",header=T,sep=";", fileEncoding="utf8") # fixed the file in rstudio and saved in utf8 encoding
dim(dipperdata)
#April_temperatures<- read.csv("April_temperatures.txt",header=T,sep=";",fileEncoding="utf8") # fixed the file in rstudio and saved in utf8 encoding

April_temperatures<- read.csv("Timing_of_ice_cover_break-up.txt",header=T,sep="",fileEncoding="utf8") # Timing of ice break up (in days after the 1st of March). Data not available from 2010 to 2013 -> values set to 46 (mean)
colnames(April_temperatures)<-c("Ar","April_temp")
lm(April_temp ~ Ar,data=April)
dipperdata<-merge(dipperdata,April_temperatures,by=c("Ar"))

completedata <- read.csv("Complete data set_with territory.txt",header=T,sep=";",fileEncoding="utf8")

#thresholds <- date.threshold(plot = FALSE)
#names(thresholds) <- c("Ar","date")
#thresholds <- rbind(thresholds,data.frame(Ar=1991, date=100))
#thresholds <- thresholds[order(thresholds$Ar),]

#dipperdata <- merge(dipperdata,thresholds, by="Ar")

# Load package/functions
library(TMB) 

# compile and load the template
compile("dipper_tmb.cpp")
dyn.load(dynlib("dipper_tmb"))

# Transform the data into TMB format 
data <- with(dipperdata,list( 
  j = factor(F_ring),                                            # female id
  k = factor(Lok),                                               # territory id
  t = factor(Ar),                                                # year id
  z = laying,                                                    # laying date
  y = cbind(Egg_ok, Unger_ok, M_F),                              # matrix containing clutch size, fledglings, recruits
  temp = temp-mean(tapply(temp,Ar,mean)),                        # winter temperature
  April_temp = April_temp-mean(tapply(April_temp,Ar,mean),na.rm=T),      # Timing of ice break up since 1st of March
  dens = N-mean(tapply(N,Ar,mean)),                              # local population density
  altitude = HoH-mean(HoH),                                      # altitude
  tt = Ar-mean(Ar),                                              # year
  a = factor(Age,levels=1:10),                                   # age of mother
  offset = log(c(22,365-22, 365)/365),                           # offsets for each episode (log of proportion of one year)
  survival = Surv,                                               # Survival (0 or 1),
  brood = Double_clutch,                                         # Double_clutch specifies whether the breeding female is present (2) or not (1) twice in the dataset a given year (double clutch) - 4 females are in that case.
  eggs = Egg_ok_year                                             # Remains to be seen if this should be centered or not
))


# Set up starting values for the parameters / parameter values under restricted models
parameters <- list(
  # some parameters may differ between selection episodes (possibly make them equal via map-argument later on)
  beta_0          = c(0,0,0),       # log hazard intercepts
  beta_age        = c(0,0,0),       # linear effect of age, not estimated if RW2 is turned on (log_nu non-missing in map)
  beta_clutchsize = c(0,0,0),       # effect of clutch size
  beta_eggs       = c(0,0,0),       # effect of total # eggs
  beta_temp       = c(0,0,0),       # effect of temp
  beta_dens       = c(0,0,0),       # effect of density
  beta_altitude   = c(0,0,0),       # effect of altitude
  beta_clutchsize_layingdate = c(0,0,0), # interaction between layingdate and clutchsize on maximum survival
  u = matrix(0,nlevels(data$t),2),  # random effect of year on survival and optimal layingdate
  v = rep(0,nlevels(data$k)),       # permanent random effect of territory on overall hazard
  w = rep(0,nlevels(data$j)),       # effect of female id on hazard (individual heterogeneity)
  x = matrix(0,nlevels(data$a),3),  # effects of different ages
  log_sigma = rep(-2,3),            # log standard deviations of random effects, possibly different for each episode
  log_tau   = rep(-2,3),
  log_kappa = rep(-2,3),            # female id random effect on survival would be confounded with age dep. surv. rates so set this sd to a small value
  log_gamma = rep(-2,2),
  log_nu    = rep(-2,3),
  theta_0          = c(45,45,45),   # mean optimal laying date intercept(s)
  theta_temp       = c(0,0,0),      # effect of temp on optimal laying date
  theta_April_temp = c(0,0,0),      # effect of April temp on optimal laying date
  theta_dens       = c(0,0,0),      # effect of density 
  theta_altitude   = c(0,0,0),      # effect of altitude 
  theta_tt   = c(0,0,0),            # effect of years (deterministic trend in optimal egg laying date) 
  theta_clutchsize = c(0,0,0),      # effect of clutchsize on theta
  log_sd_zeta = log(10),            # log standard deviations of zeta
  
  log_omega = rep(log(150),3),      # log width of optimal laying date window (in terms of the hazard)
  
  phi = c(0,0,0,0),  # elements of Phi
  rho = 0
)

# --- new way of fitting different model restrictions using update in functions.R ---
map <- list(
  beta_0 = factor(c(NA,NA,NA)),       # fix first level at zero and estimate intercepts episodes 2 and 3
  beta_age = factor(c(NA,NA,NA)),
  beta_clutchsize = factor(c(1,2,NA)),# single effect of clutchsize on offspring survival episodes
  beta_eggs = factor(c(NA,NA,1)),     # single effect of total number of eggs on mother survival: test for female cost of reproduction
  beta_temp = factor(c(NA,1,2)),
  beta_dens = factor(c(1,2,3)),
  beta_altitude = factor(c(1,2,3)),
  beta_clutchsize_layingdate = factor(c(NA,NA,NA)),
  theta_0 = factor(c(1,1,NA)),        # use theta_0[3]=NA to signal removing stab. sel part for mother survival in template
  theta_dens = factor(c(1,1,NA)),
  theta_temp = factor(c(1,1,NA)),
  theta_April_temp = factor(c(NA,NA,NA)),
  theta_altitude = factor(c(1,1,NA)),
  theta_tt   = factor(c(NA,NA,NA)),            
  theta_clutchsize = factor(c(NA,NA,NA)),
  log_sigma = factor(rep(NA,3)),
  v = factor(rep(NA,nlevels(data$k))), log_tau = factor(rep(NA,3)), # By removing this line, we include the random effect
  log_kappa = factor(rep(NA,3)), log_gamma = factor(c(1,2)),
  #log_nu = factor(rep(NA,3)),
  log_sd_zeta = factor(NA),
  log_omega = factor(c(1,2,NA)),
  phi = factor(rep(NA,4)),
  rho = factor(NA)
)

# dummy model to get things started
model00 <- list(parameters=parameters, map=map, 
                args.MakeADFun=list(data = data, DLL = "dipper_tmb", 
                                    random = c("u","v","w","x"),
                                    silent = FALSE),
                args.optim=list(method="BFGS",hessian=TRUE,control=list(maxit=2000), control=list(reltol=1e-12)),
                optimizer="optim")
class(model00) <- "tmbmodel"

# only age random effects, converges but nu[1] and nu[2] becomes very small, i.e. essentially linear trends
# aic = 2798.115  
model0 <- update(model00)  
model0


models <- list()
simodels <- list()
#################### aic =  2757.619 
models$best <- simodels$best <- update(model0, 
                                       list(log_nu = factor(c(NA,NA,1)), 
                                            beta_0 = factor(c(1,2,NA)),   
                                            beta_age = factor(c(NA,NA,NA)),
                                            u = NULL, log_sigma = factor(c(NA,1,1)),
                                            v = NULL, log_tau=factor(c(1,1,1)),
                                            beta_dens = factor(c(NA,NA,1)),
                                            beta_temp = factor(c(NA,NA,1)),
                                            beta_altitude = factor(c(1,1,NA)), # the new model includes the same altitude effect on the first two episodes
                                            beta_eggs = factor(c(NA,NA,NA)),
                                            log_omega = factor(c(1,1,NA)),
                                            theta_0 = factor(c(1,2,NA)), 
                                            theta_temp = factor(c(NA,1,NA)),
                                            theta_altitude = factor(c(1,1,NA)),
                                            theta_tt = factor(c(1,1,NA))),
                                       cv = .1, seed=2,
                                       description = "Best model, see Table \\ref{table:Table 2}")
models$best
#simodels$best <- models$best
parametertable(models$best,file="tab-bestmodel.tex")

# Linear trend for episodes 1 and 2
models$m1 <- update(models$best, 
                    list(beta_age=factor(c(1,2,NA))),
                    cv=.1,seed=1,
                    description = "$\\beta_{\\text{age},1}\\neq 0, \\beta_{\\text{age},2}\\neq 0$ (different linear age effect for episodes 1 and 2)")

models$m2 <- update(models$best, 
                    list(beta_age=factor(c(1,NA,NA))),
                    cv=.1,seed=1,
                    description = "$\\beta_{\\text{age},1}\\neq 0$ (linear age effect for episode 1)")
models$m2

models$m3 <- update(models$best, 
                    list(beta_age=factor(c(1,1,NA))),
                    cv=.1,seed=1,
                    description = "$\\beta_{\\text{age},1}=\\beta_{\\text{age},2}\\neq 0$ (common linear age effect for episodes 1 and 2)")
models$m3

models$m4 <- update(models$best, 
                    list(beta_age=factor(c(NA,2,NA))),
                    cv=.1,seed=1,
                    description = "$\\beta_{\\text{age},2}\\neq 0$ (linear age effect for episode 2)")
models$m4

# Add effect of # eggs on episode 3
models$m5 <- update(models$best, 
                    list(beta_eggs = factor(c(NA,NA,1))),
                    cv=.1,seed=1,
                    description = "$\\beta_{\\text{\\#eggs},3}$")
models$m5

# Remove effect of clutch size on episode 2
# aic =  2757.534 -> Best than the best model but we keep the best model
#models$m5a <- update(models$best,
#                     list(beta_clutchsize = factor(c(1,NA,NA))),
#                     description = "$\\beta_{\\text{clutchsize},2}=0$")
#models$m5a

# Remove effect of clutch size on episode 1
# aic = 2759.98
#models$m5b <- update(models$best,
#                     list(beta_clutchsize = factor(c(NA,1,NA))),
#                     description = "$\\beta_{\\text{clutchsize},1}=0$")
#models$m5b


# Test for u AR1
# aic =  2757.618 -> same AIC than the best model
models$m6 <- update(models$best, 
                    list(phi=factor(c(1,NA,NA,NA))),
                    cv=.1,seed=1,
                    description = "$\\phi_{11}\\neq 0$ ($u_t \\sim AR(1)$)")
models$m6


# Correlational selection
models$m8 <- update(models$best, 
                    list(beta_clutchsize_layingdate=factor(c(1,2,NA))),
                    cv=.1,seed=2,
                    description = "$\\beta_{\\text{clutchsize/date},1}\\neq 0,\\beta_{\\text{clutchsize/date},2}\\neq 0$ (correlational selection of first kind)")
models$m8
simodels$m8<-models$m8

models$m9 <- update(models$best, 
                    list(beta_clutchsize_layingdate=factor(c(1,1,NA))),
                    cv=.1,seed=1,
                    description = "$\\beta_{\\text{clutchsize/date},1}=\\beta_{\\text{clutchsize/date},2}\\neq 0$ (correlational selection of first kind)")
models$m9

# Remove stab.sel.  Remember to turn of all parameters inside quadratic stab. sel. term.
# If any such parameter is still there the model will not converge
models$m10 <- update(models$best, 
                     list(
                       theta_0 = factor(c(NA,NA,NA)), 
                       theta_dens=factor(c(NA,NA,NA)),
                       theta_altitude=factor(c(NA,NA,NA)),
                       theta_temp=factor(c(NA,NA,NA)),
                       log_omega=factor(c(NA,NA,NA))),
                     cv=.1,seed=1,
                     description = "Remove stab. sel. on $z$ (quadratic term in eq. \\ref{eq:eta-beta-theta}) for episodes 1 and 2")

models$m10

# Different omegas for the first two episodes
models$m11 <- update(models$best, 
                     list(log_omega = factor(c(1,2,NA))),
                     cv=.1,seed=2,
                     description = "$\\omega_{1}\\neq \\omega_{2}$ (strength of stabilizing selection different for episodes 1 and 2)")
models$m11

# Different theta_0 for the first two episodes
#models$m12 <- update(models$best,
#                     list(theta_0 = factor(c(1,2,NA))),
#                     cv=.1,seed=1,
#                     description = "$\\theta_{0, 1}=\\theta_{0, 2}$ (different optimal laying date for episodes 1 and 2)")
#models$m12
#simodels$m12<-models$m12

# Add theta_0 for episode 3
models$m13 <- update(models$best, 
                     list(theta_0 = factor(c(1,1,2))),
                     cv=.1,seed=1,
                     description = "$\\theta_{0, 3}\\neq 0$ (stabilizing selection for episode 3)")
models$m13


# Remove linear trend in optimal laying dates
models$m14 <- update(models$best,
                     list(theta_tt=factor(c(NA,NA,NA))),
                     cv=.1, seed=1,
                     description = "$\\theta_t=0$ (remove trend in optimal laying date)")
models$m14


# Add same theta_clutchsize on the first two episodes
models$m15 <- update(models$best, 
                     list(theta_clutchsize=factor(c(1,1,NA))),
                     cv=.1,seed=1,
                     description = "$\\theta_{\\text{clutchsize},1}=\\theta_{\\text{clutchsize},2}\\neq 0$ (correlational selection of second kind)")
models$m15


# Add zeta
models$m16 <- update(models$best,
                     list(log_sd_zeta = factor(1)),
                     cv=.1,seed=1,
                     description = "$\\sigma_\\zeta>0$ (random effect on optimal laying date)")
models$m16

# Add zeta AR1
models$m17 <- update(models$best,
                     list(phi=factor(c(NA,NA,NA,1)),
                          log_sd_zeta = factor(1)),
                     cv=.1,seed=1,
                     description = "$\\sigma_\\zeta>0$ and $\\phi_{22}\\neq 0$ ($\\zeta_t \\sim AR(1)$)")
models$m17

# Correlated optimas
models$m18 <- update(models$best,
                     list(phi=factor(c(NA,NA,1,NA)),
                          log_sd_zeta = factor(1)),
                     cv=.1,seed=1,
                     description = "$\\sigma_\\zeta > 0$ and $\\phi_{21}\\neq 0$ ($\\zeta_t$ dependent on $u_{t-1}$)")
models$m18
models$m7 <- update(models$best, 
                    list(list(phi=factor(c(NA,1,NA,NA))),
                         log_sd_zeta = NULL),
                    cv=.1,seed=1,
                    description = "$\\sigma_\\zeta > 0$ and $\\phi_{12}\\neq 0$ ($u_t$ dependent on $\\zeta_{\\text{t-1}}$)")
model$m7

# Remove effect of winter temp on optimal laying date
models$m19 <- update(models$best, 
                     list(theta_temp = factor(c(NA,NA,NA))),cv=.05,seed=2,
                     description = "$\\theta_{\\text{temp},2}=0$ (remove temperature effect on optimal laying date)")
models$m19

# Add theta_temp on the first two episodes 
# aic = 2757.035   - Best than the best model but biologically not really meaningful?
#models$m19a <- update(models$best, 
#                    list(theta_temp = factor(c(1,1,NA))),
#                    cv=.05,seed=2,
#                    description = "$\\theta_{\\text{temp},1}=\\theta_{\\text{temp},2}\\neq 0$")
#models$m19a


# Remove effect of density on optimal laying date
models$m20 <- update(models$best, 
                     list(theta_dens = factor(c(NA,NA,NA))),cv=.05,seed=2,
                     description = "$\\theta_{\\text{dens},1}=\\theta_{\\text{dens},2}=0$ (remove density effect on optimal laying date)")
models$m20


# Remove effect of altitude on optimal laying date
models$m21 <- update(models$best, 
                     list(theta_altitude = factor(c(NA,NA,NA))),cv=.05,seed=2,
                     description = "$\\theta_{\\text{alt},1}=\\theta_{\\text{alt},2}=0$ (remove altitude effect on laying date)")
models$m21

aictable(models)
aictable(models,file="tab-aic-models.tex",showkeys=FALSE) # remove showkeys in final version

simodels$m4a <- update(models$best, 
                       list(log_nu = factor(c(1,2,3)), 
                            beta_0 = factor(c(NA,NA,NA)),
                            beta_age = factor(c(NA,NA,NA))),
                       description = "All age effects modelled as second order random walks")
simodels$m4a


# Remove effect of clutch size
simodels$m5c <- update(models$best,
                       list(beta_clutchsize = factor(c(NA,NA,NA))),
                       description = "$\\beta_{\\text{clutchsize},1}=\\beta_{\\text{clutchsize},2}=0$")
simodels$m5c


# Remove effect of temp on episode 3
simodels$m5d <- update(models$best,
                       list(beta_temp = factor(c(NA,NA,NA))),
                       description = "$\\beta_{\\text{temp},3}=0$")
simodels$m5d 

# Add effect of temp on episode 2
simodels$m5e <- update(models$best,
                       list(beta_temp = factor(c(NA,1,2))),
                       description = "$\\beta_{\\text{temp},2}\\neq 0$")
simodels$m5e

# Add same effect of temp on episodes 2 and 3
simodels$m5f <- update(models$best,
                       list(beta_temp = factor(c(NA,1,1))),
                       description = "$\\beta_{\\text{temp},2}=\\beta_{\\text{temp},3}$")
simodels$m5f

# Remove density effect 
simodels$m5g <- update(models$best,
                       list(beta_dens = factor(c(NA,NA,NA))),
                       description = "$\\beta_{\\text{dens},3}=0$")
simodels$m5g 


# Add effect of density on episode 2
simodels$m5h <- update(models$best,
                       list(beta_dens = factor(c(NA,1,2))),
                       description = "$\\beta_{\\text{dens},2}\\neq 0$")
simodels$m5h

# Add effect of density on episode 1
simodels$m5hh <- update(models$best,
                        list(beta_dens = factor(c(1,NA,2))),
                        description = "$\\beta_{\\text{dens},1}\\neq 0$")
simodels$m5hh

# Add effect of density on episode 1 only
simodels$m5hhh <- update(models$best,
                         list(beta_dens = factor(c(1,NA,NA))),
                         description = "$\\beta_{\\text{dens},1}\\neq 0$, $\\beta_{\\text{dens},3}=0$")
simodels$m5hhh


# Add same effect of density on episodes 1 and 3
simodels$m5h0 <- update(models$best,
                        list(beta_dens = factor(c(1,NA,1))),
                        description = "$\\beta_{\\text{dens},1}=\\beta_{\\text{dens},3}\\neq 0$")
simodels$m5h0


# Add same effect of density on episodes 2 and 3
simodels$m5i0 <- update(models$best,
                        list(beta_dens = factor(c(NA,1,1))),
                        description = "$\\beta_{\\text{dens},2}=\\beta_{\\text{dens},3}$")
simodels$m5i0

# Add same effect of density on episode 2 only
simodels$m5i <- update(models$best,
                       list(beta_dens = factor(c(NA,1,NA))),
                       description = "$\\beta_{\\text{dens},2}\\neq 0,\\beta_{\\text{dens},3}=0$")
simodels$m5i

# Add same effect of density on all episodes
simodels$m5ii <- update(models$best,
                        list(beta_dens = factor(c(1,1,1))),
                        description = "$\\beta_{\\text{dens},1}=\\beta_{\\text{dens},2}=\\beta_{\\text{dens},3}$")
simodels$m5ii

# Add different effects of density on all episodes
simodels$m5iii <- update(models$best,
                         list(beta_dens = factor(c(1,2,3))),
                         description = "$\\beta_{\\text{dens},1}\\neq 0,\\beta_{\\text{dens},2}\\neq 0,\\beta_{\\text{dens},3}\\neq 0$")
simodels$m5iii


# Remove altitude effect 
simodels$m5j <- update(models$best,
                       list(beta_altitude = factor(c(NA,NA,NA))),
                       description = "$\\beta_{\\text{alt},1}=\\beta_{\\text{alt},2}=0$")
simodels$m5j 


# Add effect of altitude on episode 3 and remove effect on episode 1
simodels$m5k <- update(models$best,
                       list(beta_altitude = factor(c(NA,1,2))),
                       description = "$\\beta_{\\text{alt},1}=0,\\beta_{\\text{alt},3}\\neq 0$")
simodels$m5k

# Add different effects of alt on episodes 1 and 2
simodels$m5l <- update(models$best,
                       list(beta_altitude = factor(c(1,2,NA))),
                       description = "$\\beta_{\\text{alt},1}\\neq 0,\\beta_{\\text{alt},2}\\neq 0$")
simodels$m5l

# Add effect on episode 3
simodels$m5ll <- update(models$best,
                        list(beta_altitude = factor(c(1,1,2))),
                        cv=.1,seed=1,
                        description = "$\\beta_{\\text{alt},3}\\neq 0$")
ssimodels$m5ll

# Add effect of alt on episode 1 only
simodels$m5m <- update(models$best,
                       list(beta_altitude = factor(c(1,NA,NA))),
                       description = "$\\beta_{\\text{alt},2}=0$")
simodels$m5m


# Add effect of alt on episode 2 only
simodels$m5n <- update(models$best,
                       list(beta_altitude = factor(c(NA,1,NA))),
                       description = "$\\beta_{\\text{alt},1}=0$")
simodels$m5n

# Add same effect of alt on episodes 2 and 3 and remove effect on episode 1
simodels$m5o <- update(models$best,
                       list(beta_altitude = factor(c(NA,1,1))),
                       description = "$\\beta_{\\text{alt},1}=0,\\beta_{\\text{alt},2}=\\beta_{\\text{alt},3}\\neq 0$")
simodels$m5o

# Add effect of alt on episode 3 only
simodels$m5p <- update(models$best,
                       list(beta_altitude = factor(c(NA,NA,1))),
                       description = "$\\beta_{\\text{alt},1}=\\beta_{\\text{alt},2}=0,\\beta_{\\text{alt},3}\\neq 0$")
simodels$m5p

# Add same effect of altitude on all episodes
simodels$m5q <- update(models$best,
                       list(beta_altitude = factor(c(1,1,1))),
                       description = "$\\beta_{\\text{alt},1}=\\beta_{\\text{alt},2}=\\beta_{\\text{alt},3}$")
simodels$m5q

# Add different effects of density on all episodes
simodels$m5r <- update(models$best,
                       list(beta_altitude = factor(c(1,2,3))),
                       description = "$\\beta_{\\text{alt},1}\\neq 0,\\beta_{\\text{alt},2}\\neq 0,\\beta_{\\text{alt},3}\\neq 0$")
simodels$m5r




simodels$m9a <- update(models$best, 
                       list(beta_clutchsize_layingdate=factor(c(1,NA,NA))),
                       cv=.1,seed=1,
                       description = "$\\beta_{\\text{clutchsize/date},1}\\neq 0$ (correlational selection of first kind)")
simodels$m9a

simodels$m9b <- update(models$best, 
                       list(beta_clutchsize_layingdate=factor(c(NA,1,NA))),
                       cv=.1,seed=1,
                       description = "$\\beta_{\\text{clutchsize/date},2}\\neq 0$ (correlational selection of first kind)")
simodels$m9b



# Add different theta_0 for episode 3
simodels$m13a <- update(models$best, 
                        list(theta_0 = factor(c(1,2,3))),
                        cv=.1,seed=1,
                        description = "$\\theta_{0, 1}\\neq 0,\\theta_{0, 2}\\neq 0,\\theta_{0, 3}\\neq 0$")
simodels$m13a


# Different linear trends for episodes 1 and 2
simodels$m14a <- update(models$best, 
                        list(theta_tt = factor(c(1,2,NA))), 
                        cv=.1, seed=2,
                        description = "$\\theta_{t,1}\\neq \\theta_{t,2}$")
simodels$m14a


# Add different theta_clutchsize on the first two episodes
simodels$m15a <- update(models$best, 
                        list(theta_clutchsize=factor(c(1,2,NA))),
                        cv=.1,seed=1,
                        description = "$\\theta_{\\text{clutchsize},1}\\neq 0$,$\\theta_{\\text{clutchsize},2}\\neq 0$ (correlational selection of second kind)")
simodels$m15a


# Remove effect of density on optimal laying date on episode 1
simodels$m20a <- update(models$best, 
                        list(theta_dens = factor(c(NA,1,NA))),cv=.05,seed=2,
                        description = "$\\theta_{\\text{dens},1}=0$")
simodels$m20a

# Remove effect of density on optimal laying date on episode 2
simodels$m20b <- update(models$best, 
                        list(theta_dens = factor(c(1,NA,NA))),cv=.05,seed=2,
                        description = "$\\theta_{\\text{dens},2}=0$")
simodels$m20b

# Different effect of density on optimal laying date on episodes 1 and 2
simodels$m20c <- update(models$best, 
                        list(theta_dens = factor(c(1,2,NA))),cv=.05,seed=2,
                        description = "$\\theta_{\\text{dens},1}\\neq 0,\\theta_{\\text{dens},2}\\neq 0$")
simodels$m20c



# Remove effect of altitude on optimal laying date on episode 1
simodels$m21a <- update(models$best, 
                        list(theta_altitude = factor(c(NA,1,NA))),cv=.05,seed=2,
                        description = "$\\theta_{\\text{alt},1}=0$")
simodels$m21a

# Remove effect of altitude on optimal laying date on episode 2
simodels$m21b <- update(models$best, 
                        list(theta_altitude = factor(c(1,NA,NA))),cv=.05,seed=2,
                        description = "$\\theta_{\\text{alt},2}=0$")
simodels$m21b

# Different effect of altitude on optimal laying date on episodes 1 and 2
simodels$m21c <- update(models$best, 
                        list(theta_altitude = factor(c(1,2,NA))),cv=.05,seed=2,
                        description = "$\\theta_{\\text{alt},1}\\neq 0, \\theta_{\\text{alt},2}\\neq 0$")
simodels$m21c

# Different sigmas on the last two episodes 
simodels$m21d <- update(models$best, 
                        list(log_sigma = factor(c(NA,1,2))),cv=.05,seed=2,
                        description = "$\\sigma_{2}\\neq 0,\\sigma_{3}\\neq 0$")
simodels$m21d 

# Add a different sigma on the first episode 
simodels$m21e <- update(models$best, 
                        list(log_sigma = factor(c(1,2,2))),cv=.05,seed=2,
                        description = "$\\sigma_{1}\\neq 0,\\sigma_{2}=\\sigma_{3}\\neq 0$")
simodels$m21e 


# 3 different sigmas 
simodels$m21f <- update(models$best, 
                        list(log_sigma = factor(c(1,2,3))),cv=.05,seed=2,
                        description = "$\\sigma_{1}\\neq 0,\\sigma_{2}\\neq 0,\\sigma_{3}\\neq 0$")
simodels$m21f 

# 3 different log_tau (the best model considers 3 identical log_tau for the 3 episodes)
simodels$m21g <- update(models$best, 
                        list(log_tau = factor(c(1,2,3))),cv=.05,seed=2,
                        description = "$\\tau_{1}\\neq 0,\\tau_{2}\\neq 0,\\tau_{3}\\neq 0$")
simodels$m21g 

# Different log_tau for episodes 1 and 3
simodels$m21h <- update(models$best, 
                        list(log_tau = factor(c(1,NA,2))),cv=.05,seed=2,
                        description = "$\\tau_{1}\\neq 0,\\tau_{2}=0,\\tau_{3}\\neq 0$")
simodels$m21h 


# Same log_tau for episodes 1 and 3
# aic =  2757.456 -> Slightly better fit than the best model
simodels$m21i <- update(models$best, 
                        list(log_tau = factor(c(1,NA,1))),cv=.1,seed=2,
                        description = "$\\tau_{1}=\\tau_{3}\\neq 0$,$\\tau_{2}=0$")
simodels$m21i 


# Same log_tau for episodes 1 and 3 and different value for episode 2
simodels$m21j <- update(models$best, 
                        list(log_tau = factor(c(1,2,1))),cv=.1,seed=2,
                        description = "$\\tau_1=\\tau_3\\neq 0$, $\\tau_2\\neq 0$")
simodels$m21j 


# Add w (mother id) as random effect, all kappa's estimated 
simodels$m21k <- update(models$best, 
                        list(w = NULL, log_kappa=NULL),cv=.1,seed=2,
                        description = "$\\kappa_{1}\\neq 0$, $\\kappa_{2}\\neq 0$, $\\kappa_{3}\\neq 0$")

simodels$m21k


# Add w on episodes 2 and 3 
simodels$m21k1 <- update(models$best, 
                         list(log_kappa=factor(c(NA,1,2))),cv=.1,seed=2,
                         description = "$\\kappa_{1}=0$, $\\kappa_{2}\\neq 0$, $\\kappa_{3}\\neq 0$")

simodels$m21k1


# Add w on episode 2 
simodels$m21l <- update(models$best, 
                        list(log_kappa=factor(c(NA,1,NA))),cv=.1,seed=2,
                        description = "$\\kappa_{1}=\\kappa_{3}=0,\\kappa_{2}\\neq 0$")

simodels$m21l

# Same w on episodes 2 and 3
simodels$m21m <- update(models$best, 
                        list(log_kappa=factor(c(NA,1,1))),cv=.1,seed=2,
                        description = "$\\kappa_{2}=\\kappa_{3}\\neq 0,\\kappa_{1}=0$")

simodels$m21m


# AR1 for u and zeta
simodels$m21n <- update(models$best, 
                        list(phi=factor(c(1,NA,NA,2))),cv=.1,seed=2,
                        description = "$\\sigma_\\zeta > 0$ and $\\phi_{11}\\neq 0$ and $\\phi_{22}\\neq 0$ ($u_t$ and $\\zeta _{t}\\sim AR(1)$)")

simodels$m21n


# Correlated optimas?
simodels$m21n1 <- update(models$best, 
                         list(phi=factor(c(NA,1,2,NA))),cv=.1,seed=2,
                         description = "$\\sigma_\\zeta > 0$ and $\\phi_{12}\\neq 0$ and $\\phi_{21}\\neq 0$")

simodels$m21n1


# Correlated optimas and AR1?
simodels$m21o <- update(models$best, 
                        list(phi=factor(c(1,2,3,4))),cv=.1,seed=2,
                        description = "$\\sigma_\\zeta > 0$ and $\\phi_{11}$, $\\phi_{12}$, $\\phi_{21}$ and $\\phi_{22}\\neq 0$")

simodels$m21o

aictable(simodels)
aictable(simodels,file="tab-aic-simodels.tex",showkeys=TRUE) # remove showkeys in final version


################## In SI Appendix 1 - Spring effect

# Add effect of April temp on optimal egg laying date for the first two episodes
models$mspring1 <- update(models$best, 
                          list(theta_April_temp=factor(c(1,1,NA))),cv=.1,seed=2,
                          description = "$\\theta_{\\text{spring},1}=\\theta_{\\text{spring},2}\\neq 0$")

models$mspring1

# Add different effect of April temp on optimal egg laying date for the first two episodes
models$mspring1a <- update(models$best, 
                           list(theta_April_temp=factor(c(1,2,NA))),cv=.1,seed=2,
                           description = "$\\theta_{\\text{spring},1}\\neq 0, \\theta_{\\text{spring},2}\\neq 0$")

models$mspring1a


# Add effect of April temp on optimal egg laying date for the first episode
models$mspring1b <- update(models$best, 
                           list(theta_April_temp=factor(c(1,NA,NA))),cv=.1,seed=2,
                           description = "$\\theta_{\\text{spring},1}\\neq 0$")

models$mspring1b

# Add effect of April temp on optimal egg laying date for the first episode and remove temp effect
models$mspring1ba <- update(models$best, 
                            list(theta_April_temp=factor(c(1,NA,NA)),
                                 theta_temp=factor(c(NA,NA,NA))), cv=.1,seed=2,
                            description = "$\\theta_{\\text{spring},1}\\neq 0$")

models$mspring1ba

# Add different effect of April temp on optimal egg laying date for the second episode
models$mspring1c <- update(models$best, 
                           list(theta_April_temp=factor(c(NA,1,NA))),cv=.1,seed=2,
                           description = "$\\theta_{\\text{spring},2}\\neq 0$")

models$mspring1c


# Remove effect of winter temp on optimal egg laying date for the first two episodes (just keep April temp)
models$mspring2 <- update(models$mspring1, 
                          list(theta_temp=factor(c(NA,NA,NA))),cv=.1,seed=2,
                          description = "$\\theta_{\\text{spring},1}=\\theta_{\\text{spring},2}\\neq 0$ and $\\theta_{\\text{temp},1}=\\theta_{\\text{temp},2}=0$")

models$mspring2


########################### Do the model without the temporal environmental covariates (temp and dens) but put zeta back in aswell as autocorrelation
simple <- list()

# aic =  2813.721
(model000 <- update(models$best, 
                    list(
                      beta_temp = factor(rep(NA,3)),
                      beta_dens = factor(rep(NA,3)),
                      theta_dens = factor(rep(NA,3)),
                      theta_temp = factor(rep(NA,3)),
                      theta_tt = factor(rep(NA,3)),
                      theta_0 = factor(c(1,1,NA)),
                      log_sd_zeta = NULL),
                    description = "Remove temporal covariates"))

#(model0000 <- update(model000, list(theta_tt = factor(c(1,1,NA)))))

# aic =  2809.217
(simple$best<-update(model000, 
                     list(phi=factor(c(NA,1,NA,NA))),
                     cv=0.01,seed=5,
                     description = "Best model"))
# aic = 2815.959
(simple$noauto <- update(model000,
                         description = "$\\phi_{12}= 0$"))
# aic 2811.009
(simple$phi11 <- update(simple$best, 
                        list(phi=factor(c(2,1,NA,NA))),
                        cv=0.01,seed=5,
                        description = "$\\phi_{11}\\neq 0$"))
# aic = 2810.968
(simple$phi22 <- update(simple$best, 
                        list(phi=factor(c(NA,1,NA,2))),
                        cv=0.01,seed=4,
                        description = "$\\phi_{22}\\neq 0$"))
# aic = 2810.807
(simple$phi21 <- update(simple$best, 
                        list(phi=factor(c(NA,1,2,NA))),
                        cv=0.01,seed=3,
                        description = "$\\phi_{21}\\neq 0$"))
# aic = 2816.315
(simple$phi11diag <- update(simple$best, 
                            list(phi=factor(c(1,NA,NA,NA))),
                            cv=0.01,seed=3,
                            description = "$\\phi_{11}\\neq 0$, $\\phi_{12}=0$"))
# aic = 2814.566
(simple$phi22diag <- update(simple$noauto, 
                            list(phi=factor(c(NA,NA,NA,1))),
                            cv=0.02,seed=2,
                            description = "$\\phi_{22}\\neq 0$, $\\phi_{12}=0$"))
# aic = 2817.003  
# (simple$rho <- update(simple$best, 
#                       list(rho=NULL), 
#                       cv=.01,seed=1,
#                       description = "$\\rho\\neq 0$"))
simple$rho <- NULL

# # aic = 2816.079 -> convergence issues?
# (simple$model4 <- update(simple$best, 
#                          list(phi=factor(c(1,2,3,4))),
#                          cv=.1,seed=2,
#                          description = "$\\phi_{12}\\neq 0$,$\\phi_{21}\\neq 0$"))
simple$model4 <- NULL



aictable(simple)
aictable(simple,file="tab-aic-simplemodels.tex",showkeys=TRUE) # remove showkeys in final version
parametertable(simple$best,file="tab-simple2.tex")


############ Figure survival as a function of age - Figure 1
epicol = c("red","green","blue")
layout(cbind(c(1,2)),6,c(2.5,3))
par(mar=c(1,4,.5,.5))
plot(NA, xlim=c(1,10), ylim=c(0.2,20),  xlab="Age of breeding female", ylab="Log hazard",axes=F, log="y")
axis(2)
with(models$best$estimates,{
  ages <- 1:10
  for (s in 1:3) {
    lines(ages,exp(beta_0[s] + beta_age[s]*ages + beta_clutchsize[s]*5 + x[,s]), col=epicol[s])
  }
})
legend("topleft", legend = c("Episode 1", "Episode 2","Episode 3"), lty=1, col=epicol, bty = "n")

par(mar=c(4,4,.5,.5))
plot(NA, xlim=c(1,10), ylim=c(0,1),  xlab="Age of breeding female", ylab="Survival probabilities",axes=F)
axis(1, at=1:10)
axis(2)
with(models$best$estimates,{
  ages <- 1:10
  for (s in 1:3) {
    lines(ages,exp(-exp(beta_0[s] + beta_age[s]*ages + beta_clutchsize[s]*5 + x[,s] + data$offset[s])), col=epicol[s])
  }
})
dev.copy2pdf(file="fig-age_effect.pdf",width=4,height=6)


############ Model output and plots

# Figure 2 - Survival and recruitment as function of clutch size and egg-laying date
layout(cbind(1,2,3),c(4,3,3),3)
# Mean survival on episode 1 as function of clutch size and egg-laying date
par(mar=c(4,4,.8,0.2))
contour.survival(models$best,1)
legend("topleft",legend="A",bty="n")
# Mean survival on episode 2 as function of clutch size and egg-laying date
par(mar=c(4,.2,.8,0.2))
contour.survival(models$best,2)
legend("topleft",legend="B",bty="n")
# Mean recruitment (clutch size * offspring survival) as function of clutch size and egg-laying date
par(mar=c(4,.2,.8,0.2))
contour.recruitment(models$best)
legend("topleft",legend="C",bty="n")
dev.copy2pdf(file="fig-survival-recruitment.pdf",width=5,height=2.2)



# Figure 3 - Annual variations in observed and estimated optimal egg-laying dates AND clutch sizes
opt1 <- optimas(models$best)
opt2 <- optimas(simple$best)
fig.optimas(list(opt1))
legend(x=1990,y=110,legend = c("Observed phenotype","Optimal phenotype","Optimum for episode 1", "Optimum for episode 2"), lty=1, lwd=c(2,1,1,1),col=c("darkgrey","black", "red","green"), bty = "n",y.intersp=0.8)
dev.copy2pdf(file="fig-optimum-best.pdf",width=5,height=4)
fig.optimas(list(opt2))
dev.copy2pdf(file="fig-optimum-sibest.pdf",width=5,height=4)

# Figure X showing optimal and mean phenotypic laying date against altitude
completedata <- read.csv("Complete data set_with territory.txt",header=T,sep=";",fileEncoding="utf8")
dat_alt <- read.csv("dat_altitude.inp",header=T,sep="",fileEncoding="utf8")
completedata_alt <- merge(completedata,dat_alt, by="Lok")
p <- with(completedata_alt, table(completedata_alt$Hoh,completedata_alt$laying))
p <- p/apply(p,1,sum)
par(mfrow=c(1,1),mar=c(4,4,.2,.2))
plot(NA, xlab="Altitude",ylab="Egg-laying date",frame=F, axes=F,ylim=c(0,100),xlim=c(-250,500))
axis(1)
axis.date(2)
aaa<-sort(unique(completedata_alt$Hoh-mean(dipperdata$HoH)))
ll<-sort(unique(completedata_alt$laying))
for (j in 1:78) 
{
  points(aaa, rep(ll[j],139), pch=16, col="grey", cex=2*sqrt(p[,as.character(ll[j])]))
}
altitude_obs<-(completedata_alt$Hoh-mean(dipperdata$HoH))
abline(zgradient <- lm(completedata_alt$laying ~altitude_obs), lty=1, col="grey", lwd=2)
with(models$best$estimates, abline(a=theta_0[1],b=theta_altitude, lty=1))
legend("topleft", ncol=1, legend = c("Mean phenotype value", "Estimated optimum"), lwd=c(2,1), col=c("grey",1), lty=c(1,1), bty = "n", cex = 1)
dev.copy2pdf(file="fig-spatial-gradient.pdf",width=4.7,height=4)

# Test if the slopes are different
slope <- rbind(
  coef(summary(zgradient))[2,1:2],
  summary(models$best$sdreport,"fixed")["theta_altitude",]
)
slope
z <- diff(slope[,"Estimate"])/sqrt(sum(slope[,"Std. Error"]^2))  # Z-score
pnorm(abs(z),lower.tail=FALSE)  # p-value



## Autocorrelation of temporal covariates dens and temp
dens <- tapply(dipperdata$N,dipperdata[,1],mean)-mean(tapply(dipperdata$N,dipperdata[,1],mean))
temp <- tapply(dipperdata$temp,dipperdata[,1],mean)-mean(tapply(dipperdata$temp,dipperdata[,1],mean))
acf(cbind(dens,temp))
dev.copy2pdf(file="fig-cross-autocov.pdf",width=8,height=7)
par(mfrow=c(2,1))
for (s in 1:2) {
  acf(models$best$estimates$theta_temp[s]*temp + models$best$estimates$theta_dens[s]*dens,main=paste("episode",s))
}
dev.copy2pdf(file="fig-autocov-temp-and-dens.pdf",width=4,height=7)

sd(temp)
sd(dens)
models$best$estimates$theta_temp
models$best$estimates$theta_dens

v1 <- var(temp)*models$best$estimates$theta_temp[2]^2
v2 <- var(dens)*models$best$estimates$theta_dens[2]^2
v1/(v1+v2)

par(mfrow=c(4,1))
for (i in 2:4)
  acf(opt1$optima[,i] - models$best$estimates$theta_tt[1]*tt)
opt1$optima
